package hk.advanpro.android.sdk;

import android.content.Context;
import android.os.Environment;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import hk.advanpro.android.sdk.commons.AConfig;
import hk.advanpro.android.sdk.device.DeviceManager;

/**
 * 安润普安卓sdk开发库。
 * Created by yejianfei on 2017/6/7.
 */
@SuppressWarnings("unchecked")
public class AdvanproAndroidSDK {

    private static Context AppContext;
    private static String appSoreDir;
    private static AConfig AppConfig = new AConfig();
    private static final Map<Class<?>, Object> MANAGER_INSTANCES = new HashMap<>();

    public static void init(Context context) {
        AppContext = context;
        File folder = getAppContext().getExternalFilesDir(null);
        if (folder == null) {
            File dir = Environment.getExternalStorageDirectory();
            folder = new File(dir + "/Advanpro/");
            if (!folder.exists()) folder.mkdir();
        }
        appSoreDir = folder.getAbsolutePath();
    }

    public static void init(Context context, AConfig config) {
        init(context);
        AppConfig = config;
    }

    public static <T extends DeviceManager> T getDeviceManager(Class<T> type) {
        try {
            T o = (T) MANAGER_INSTANCES.get(type);

            if (o == null) {
                o = type.newInstance();
                o.init(AppContext);
                MANAGER_INSTANCES.put(type, o);
            }

            return o;
        } catch (InstantiationException | IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }


    public static Context getAppContext() {
        return AppContext;
    }

    public static void setConfig(AConfig config) {
        AppConfig = config;
    }

    public static AConfig getConfig() {
        return AppConfig;
    }

    /**
     * 获取数据存储位置
     *
     * @return 数据存储位置
     */
    public static String getAppSoreDir() {
        return appSoreDir;
    }

    public static void setAppSoreDir(String appSoreDir) {
        AdvanproAndroidSDK.appSoreDir = appSoreDir;
    }
}
