package hk.advanpro.android.sdk.device;

import android.content.Context;

import hk.advanpro.android.sdk.device.callback.DeviceManagerScanCallback;
import hk.advanpro.android.sdk.device.enumeration.ConnectType;
import hk.advanpro.android.sdk.device.enumeration.DeviceType;
import hk.advanpro.android.sdk.device.result.DeviceScanResult;

/**
 * 安润普开发库设备管理器，主要用于扫描、连接或实例化设备对象。
 * Created by yejianfei on 2017/6/7.
 */

public interface DeviceManager<T extends DeviceScanResult> {


    void init(Context context);


    /**
     * 进行设备扫描，蓝牙使用系统扫描机制，串口获取系统串口列表。
     *
     * @param timeout 超时时间 单位秒
     * @param cb      回调接口
     */
    void scan(int timeout, DeviceManagerScanCallback<T> cb);

    /**
     * 进行设备扫描，蓝牙使用系统扫描机制，串口获取系统串口列表。
     *
     * @param timeout    超时时间 单位秒
     * @param filterType 过滤设备类型
     * @param cb         回调接口
     */
    void scan(int timeout, DeviceType filterType, DeviceManagerScanCallback<T> cb);

    /**
     * 使用连接信息，直接连接设备
     *
     * @param connection 连接信息 默认30秒
     * @param cb         回调接口
     */
    void instance(DeviceConnection connection, DeviceManagerScanCallback<T> cb);

    /**
     * 使用连接信息，直接连接设备
     *
     * @param connection 连接信息
     * @param timeout    超时时间 单位秒
     * @param cb         回调接口
     */
    void instance(DeviceConnection connection, int timeout, DeviceManagerScanCallback<T> cb);

    /**
     * 停止设备扫描操作。
     */
    void stop();

    /**
     * 蓝牙是否用
     * @param type type
     * @return 蓝牙是否用
     */
    boolean isEnable(ConnectType type);


    /**
     * 是否正在搜索
     *
     * @return 是否正在搜索
     */
    boolean isScanning();

    class DeviceConnection {

        public DeviceConnection() {
        }

        public DeviceConnection(String address) {
            this.address = address;
        }

        private String address;

        public String getAddress() {
            return address;
        }

        public void setAddress(String address) {
            this.address = address;
        }
    }

}
