package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.TimerTask;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceUpgradeCallback;
import hk.advanpro.android.sdk.device.enumeration.DeviceMode;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.params.ble.pi.BLEAdvanproPiCommandParams;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiCommandResult;
import hk.advanpro.android.sdk.device.result.ble.pi.BLEAdvanproPiDataEventResult;
import hk.advanpro.android.sdk.utils.CommonUtils;

/**
 * Created by yejianfei on 2017/6/27.
 */

public class BLEAdvanproPiDevice extends BLEDevice {

    public static final String BLE_PI_INFO_SD_TOTAL_SIZE = "BLE_PI_INFO_SD_TOTAL_SIZE";
    public static final String BLE_PI_INFO_SD_FREE_SIZE = "BLE_PI_INFO_SD_FREE_SIZE";
    public static final String BLE_PI_INFO_STATE = "BLE_PI_INFO_STATE";
    public static final String EVENT_BATTERY_CHANGE = "EVENT_BATTERY_CHANGE";
    /**
     * 启动采样
     **/
    public static final String CMD_PI_START = "st";
    /**
     * 停止采样
     **/
    public static final String CMD_PI_STOP = "sp";
    /**
     * 在设备处于采集状态时，获取 start 配置信息
     **/
    public static final String CMD_PI_GET_START_CONFIG = "gsc";
    /**
     * 用于获取当前上发数据的列名
     **/
    public static final String CMD_PI_GET_DATA_NAME = "gdn";
    /**
     * 获取主采样频率
     **/
    public static final String CMD_PI_GET_RATE = "gr";
    /**
     * 设置主采样频率
     **/
    public static final String CMD_PI_SET_RATE = "sr";
    /**
     * 获取数据输出途径
     */
    public static final String CMD_PI_GET_OUTPUT = "go";
    /**
     * 设置数据输出途径
     **/
    public static final String CMD_PI_SET_OUTPUT = "so";
    /**
     * 获取传感器的采样分频率
     **/
    public static final String CMD_PI_GET_RATEDIV = "grd";
    /**
     * 设置传感器的采样分频率
     **/
    public static final String CMD_PI_SET_RATEDIV = "srd";
    /**
     * 获取传感器的采样分频数
     **/
    public static final String CMD_PI_GET_RANGE = "grg";
    /**
     * 设置部分传感器量程
     **/
    public static final String CMD_PI_SET_RANGE = "srg";
    /**
     * 获取系统信息
     **/
    public static final String CMD_PI_GET_INFO = "gi";
    /**
     * 获取传感器列表
     **/
    public static final String CMD_PI_GET_SENSORLIST = "gsl";
    /**
     * 设置传感器为采集态
     **/
    public static final String CMD_PI_SET_SENSORACTIVE = "ssa";
    /**
     * 获取标记为采集态的传感器列表
     **/
    public static final String CMD_PI_GET_SENSORACTIVE = "gsa";
    /**
     * 获取命令说明
     **/
    public static final String CMD_PI_HELP = "help";
    private static final String TAG = BLEAdvanproPiDevice.class.getSimpleName();
    private static final UUID PRIMARY_SERVICE_UUID = UUID.fromString("00008001-0000-1000-8000-00805F9B34FB");

    private static final UUID PRIMARY_CHARACTERISTIC_UUID = UUID.fromString("00008086-0000-1000-8000-00805F9B34FB");

    private static short num = 1;

    final private StringBuffer buffer = new StringBuffer();
    private final Map<String, BLEAdvanproPiCommandResult> cache = new HashMap<>();
    protected boolean isInfoRead;
    private int readInfoTimeout = 30;//获取设备数据超时数据，单位秒
    private boolean ignore = false;
    private String[] names;//data表头
    final private BLEDeviceBluetoothGattCallback callback = new BLEDeviceBluetoothGattCallback() {
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            super.onServicesDiscovered(gatt, status);
            if (DeviceMode.BootLoader.name().equals(getInfo(BLE_DEVICE_INFO_MODE))) {
                fireConnected(null);
            } else if (!isFindService(PRIMARY_SERVICE_UUID)) {
                //服务不存在
                fireConnected(new DeviceCallbackException(ERROR_SERVICE_NOT_EXISTS));
            } else if (!isFindCharacteristic(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID)) {
                //特征不存在
                fireConnected(new DeviceCallbackException(ERROR_CHARACTERISTIC_NOT_EXISTS));
            } else {
                BLEAdvanproPiDevice.this.notify(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID, true);

                fireConnected(null);
                readDeviceInfo();
            }
        }

        @Override
        public void onCharacteristicChanged(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            super.onCharacteristicChanged(gatt, characteristic);
            ALog.d(TAG, String.format("[%s] notify %s & %s [%s]", bluetoothDevice.getAddress(), characteristic.getService().getUuid().toString(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(characteristic.getValue())));

            if (!PRIMARY_CHARACTERISTIC_UUID.equals(characteristic.getUuid())) {
                return;
            }

            line(characteristic.getValue());
        }
    };

    @Override
    public void connect(int timeOut, DeviceConnectCallback cb) {
        isInfoRead = false;
        super.connect(timeOut, cb);
        //开启5分钟获取剩余电量定时器
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (isUpgrading() || !isConnected()) {
                    return;
                }
                command(CMD_PI_GET_INFO, new BLEAdvanproPiCommandParams("bat"), new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
                    @Override
                    public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                        String bat = result.getVal();
                        if (StringUtils.isNotEmpty(bat)) {
                            int battery = Integer.parseInt(bat);
                            info.put(BLE_DEVICE_INFO_BATTERY, String.valueOf(battery));
                            fireEvent(EVENT_BATTERY_CHANGE, null);
                        }
                    }

                    @Override
                    public void onError(Device device, DeviceCallbackException error) {
                        error.printStackTrace();
                    }
                });
            }

        }, 300 * 1000, 300 * 1000);
    }

    /**
     * 刷新设备信息
     */
    public void readDeviceInfo() {
        command(CMD_PI_GET_INFO, new BLEAdvanproPiCommandParams(), readInfoTimeout, new DeviceCommandCallback<BLEAdvanproPiCommandResult>() {
            @Override
            public void onSuccess(Device device, BLEAdvanproPiCommandResult result) {
                String dID = result.get("dID");
                String sdt = result.get("sdt(MB)");
                String sdf = result.get("sdf(MB)");
                String hw = result.get("hw");
                String fw = result.get("fw");
                String state = result.get("state");//run/stop
                String bat = result.get("bat(%)");//bat
                String pID = result.get("pID");//pID C10B41
                info.put(BLE_DEVICE_INFO_ID, dID);
                info.put(BLE_DEVICE_INFO_HARDWARE, hw);
                info.put(BLE_DEVICE_INFO_FIRMWARE, fw);
                info.put(BLE_PI_INFO_SD_TOTAL_SIZE, sdt);
                info.put(BLE_PI_INFO_SD_FREE_SIZE, sdf);
                info.put(BLE_PI_INFO_STATE, state);
                info.put(BLE_DEVICE_INFO_CODE, "");
                info.put(BLE_DEVICE_INFO_BATTERY, bat);
                isInfoRead = true;
                fireEvent(EVENT_READ_INFO, null);
            }

            @Override
            public void onError(Device device, DeviceCallbackException error) {
                fireEvent(EVENT_READ_INFO_ERROR, null);
            }
        });
    }

    public void setReadInfoTimeout(int readInfoTimeout) {
        this.readInfoTimeout = readInfoTimeout;
    }

    private void line(byte[] data) {
        if (data == null || data.length == 0) {
            return;
        }

        synchronized (buffer) {
            int index = ArrayUtils.indexOf(data, (byte) 0x0a);
            if (index >= 0) {
                parse(buffer.toString() + new String(Arrays.copyOfRange(data, 0, index)));
                buffer.delete(0, buffer.length());

                line(Arrays.copyOfRange(data, index + 1, data.length));

            } else {
                buffer.append(new String(data));
            }
        }

    }

    private void parse(String str) {
        ALog.d(TAG, String.format("[%s] parse string: %s", bluetoothDevice.getAddress(), str));
        Pattern pattern = Pattern.compile("^^\\[(\\d+)] (input|head|name|data|val|result|end)((\\t(.+))*)(\\r*)$");
        Matcher matcher = pattern.matcher(str);
        if (!matcher.find()) {
            return;
        }

        synchronized (cache) {
            String number = matcher.group(1);
            String key = matcher.group(2);

            if ("name".equals(key)) {
                names = matcher.group(3) != null
                        ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                return;
            }

            if ("data".equals(key)) {
                String[] datas = matcher.group(3) != null
                        ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                fireEvent(EVENT_DEVICE_DATA,
                        new BLEAdvanproPiDataEventResult(Integer.parseInt(number), names, datas));
                return;
            }

            BLEAdvanproPiCommandResult result = cache.get(number);
            if (result == null) {
                return;
            }

            switch (key) {
                case "input":
                    break;
                case "head":
                    String[] heads = matcher.group(3) != null
                            ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                    result.setHeads(heads);
                    break;
                case "val":
                    String[] vals = matcher.group(3) != null
                            ? StringUtils.split(matcher.group(3).trim(), '\t') : new String[0];
                    if (vals.length == 1) {
                        result.setVal(vals[0]);
                    } else {
                        result.setVals(vals);
                    }
                    break;
                case "name":
                    break;
                case "data":
                    break;
                case "result":
                    result.setCode(Integer.parseInt(matcher.group(3).replace("[", "").replace("]", "").trim()));
                    break;
                case "end":
                    synchronized (cache) {
                        cache.remove(number);
                    }
                    beforeFireCommand(number, result);
                    fireCommand(number, result);
                    break;
                default:
            }

        }

    }

    protected void beforeFireCommand(String number, BLEAdvanproPiCommandResult result) {
        switch (result.getKey()) {
            case CMD_PI_GET_DATA_NAME:
                names = result.getVals();
                break;
            case CMD_PI_START:
                info.put(BLE_PI_INFO_STATE, "run");
                break;
            case CMD_PI_STOP:
                info.put(BLE_PI_INFO_STATE, "stop");
                break;
            default:
                break;
        }
    }

    @Override
    public synchronized void command(String key, DeviceCommandParams params, int timeOut, DeviceCommandCallback cb) {
        if (num == Short.MAX_VALUE) {
            num = 0;
        }
        short n = num++;

        BLEAdvanproPiCommandParams cp = (BLEAdvanproPiCommandParams) params;

        cp.setKey(key);
        cp.setCmdNumber(n);
        cache.put(String.valueOf(n), new BLEAdvanproPiCommandResult(key, n));

        BLEDevice.CommandChannelInfo info = new BLEDevice.CommandChannelInfo(String.valueOf(n), PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID);

        if (registerCommand(info, timeOut, cb)) {
            write(PRIMARY_SERVICE_UUID, PRIMARY_CHARACTERISTIC_UUID, params.toBytes());
        }
    }

    @Override
    public void start(DeviceCommandParams params, DeviceCommandCallback cb) {
        command(CMD_PI_START, params, cb);

    }

    @Override
    public void stop(DeviceCommandParams params, DeviceCommandCallback cb) {
        command(CMD_PI_STOP, params, cb);

    }

    @Override
    public void settings(DeviceCommandParams params, DeviceCommandCallback cb) {

    }

    @Override
    public boolean upgrade(File file, DeviceUpgradeCallback cb) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean isUpgrading() {
        return false;
    }

    @Override
    public boolean isInfoRead() {
        return isInfoRead;
    }

    @Override
    protected BluetoothGattCallback getBluetoothGattCallback() {
        return callback;
    }

    @Override
    protected CommandChannelInfo getCommandChannelInfo(String key) {
        return null;
    }

    @Override
    protected void afterRegisterCommand(String key, DeviceCommandCallback cb) {

    }

    @Override
    protected void onCommandTimeOut(String key) {
        synchronized (cache) {
            cache.remove(key);
        }
    }

}
