package hk.advanpro.android.sdk.device.ble;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothProfile;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.support.annotation.NonNull;
import android.util.Log;

import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.PriorityBlockingQueue;

import hk.advanpro.android.sdk.AdvanproAndroidSDK;
import hk.advanpro.android.sdk.commons.ALog;
import hk.advanpro.android.sdk.device.Device;
import hk.advanpro.android.sdk.device.callback.DeviceCallbackException;
import hk.advanpro.android.sdk.device.callback.DeviceCommandCallback;
import hk.advanpro.android.sdk.device.callback.DeviceConnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceDisconnectCallback;
import hk.advanpro.android.sdk.device.callback.DeviceEventCallback;
import hk.advanpro.android.sdk.device.params.DeviceCommandParams;
import hk.advanpro.android.sdk.device.result.DeviceCommandResult;
import hk.advanpro.android.sdk.device.result.DeviceEventResult;
import hk.advanpro.android.sdk.device.result.DeviceScanResult;
import hk.advanpro.android.sdk.device.result.ble.BLEConnectStateChangeResult;
import hk.advanpro.android.sdk.device.result.ble.BLEDeviceScanResult;
import hk.advanpro.android.sdk.utils.CommonUtils;

/**
 * Created by yejianfei on 2017/6/27.
 */

abstract public class BLEDevice implements Device {
    /**
     * The profile is in disconnected state
     */
    public static final int BLE_CONNECT_STATE_DISCONNECTED = 0;
    /**
     * The profile is in connecting state
     */
    public static final int BLE_CONNECT_STATE_CONNECTING = 1;
    /**
     * The profile is in connected state
     */
    public static final int BLE_CONNECT_STATE_CONNECTED = 2;
    /**
     * The profile is in disconnecting state
     */
    public static final int BLE_CONNECT_STATE_DISCONNECTING = 3;
    public static final int DISCOVER_SERVICE_RETRY_TIME = 3;//发现服务重试次数
    public static final String BLE_DEVICE_INFO_NAME = "BLE_DEVICE_INFO_NAME";
    public static final String BLE_DEVICE_INFO_ADDRESS = "BLE_DEVICE_INFO_ADDRESS";
    public static final String BLE_DEVICE_INFO_TYPE = "BLE_DEVICE_INFO_TYPE";
    public static final String BLE_DEVICE_INFO_VERSION = "BLE_DEVICE_INFO_VERSION";
    public static final String BLE_DEVICE_INFO_MODE = "BLE_DEVICE_INFO_MODE";
    public static final String BLE_DEVICE_INFO_COMPANY = "BLE_DEVICE_INFO_COMPANY";
    public static final String BLE_DEVICE_INFO_SERIAL = "BLE_DEVICE_INFO_SERIAL";
    public static final String BLE_DEVICE_INFO_MAJOR = "BLE_DEVICE_INFO_MAJOR";
    public static final String EVENT_DEVICE_DATA = "EVENT_DEVICE_DATA";
    public static final String BLE_DEVICE_INFO_ID = "BLE_BLE_DEVICE_INFO_ID";
    public static final String BLE_DEVICE_INFO_CODE = "BLE_DEVICE_INFO_CODE";
    public static final String BLE_DEVICE_INFO_FIRMWARE = "BLE_DEVICE_INFO_FIRMWARE";
    public static final String BLE_DEVICE_INFO_HARDWARE = "BLE_DEVICE_INFO_HARDWARE";
    public static final String BLE_DEVICE_INFO_BATTERY = "BLE_DEVICE_INFO_BATTERY";
    public static final String EVENT_READ_INFO = "EVENT_READ_INFO";
    public static final String EVENT_READ_INFO_ERROR = "EVENT_READ_INFO_ERROR";
    public static final String EVENT_CONNECT_STATE_CHANGE = "EVENT_CONNECT_STATE_CHANGE";
    protected static final UUID CLIENT_CHARACTERISTIC_CONFIG = UUID.fromString("00002902-0000-1000-8000-00805f9b34fb");
    private static final String TAG = BLEDevice.class.getSimpleName();
    protected final Map<String, String> info = new HashMap<>();
    private final Map<String, Set<DeviceEventCallback>> listeners = new HashMap<>();
    private final Map<String, DeviceCommandCallback> commandCallbackMap = new HashMap<>();
    private final Queue<BLEDevice.SerialQueueItem> queue = new PriorityBlockingQueue<>();
    private final ExecutorService eventExecutor = Executors.newFixedThreadPool(5);
    private final ExecutorService commandExecutors = Executors.newFixedThreadPool(5);
    /**
     * 服务发现中
     */
    protected boolean isServiceDiscovering;
    protected BLEDeviceScanResult result;
    protected BluetoothDevice bluetoothDevice;
    protected BluetoothGatt gatt;
    protected Timer timer;
    protected boolean enableReconnect = true;//自动断开连接后是否自动重连
    protected int state = 0;
    private DeviceConnectCallback connectCallback;
    private DeviceDisconnectCallback disconnectCallback;


    private BroadcastReceiver broadcastReceiver;//蓝牙广播监听

    public BLEDevice() {
    }

    @Override
    public synchronized void open(DeviceScanResult result) {
        this.result = (BLEDeviceScanResult) result;
        this.bluetoothDevice = this.result.getBluetoothDevice();

        BLEDeviceScanResult.ScanRecordManufacturer manufacturer = this.result.getRecord().getManufacturer();

        this.info.put(BLE_DEVICE_INFO_NAME, this.bluetoothDevice.getName());
        this.info.put(BLE_DEVICE_INFO_ADDRESS, this.result.getBluetoothDevice().getAddress());
        this.info.put(BLE_DEVICE_INFO_TYPE, String.valueOf(manufacturer.getType()));
        this.info.put(BLE_DEVICE_INFO_VERSION, String.valueOf(manufacturer.getVersion()));
        this.info.put(BLE_DEVICE_INFO_MODE, String.valueOf(manufacturer.getMode()));
        this.info.put(BLE_DEVICE_INFO_COMPANY, String.valueOf(manufacturer.getCompany()));
        this.info.put(BLE_DEVICE_INFO_SERIAL, String.valueOf(manufacturer.getSerial()));
        this.info.put(BLE_DEVICE_INFO_MAJOR, String.valueOf(manufacturer.getMajor()));

    }

    @Override
    public synchronized void connect(DeviceScanResult result, DeviceConnectCallback cb) {
        this.open(result);
        this.connect(cb);
    }


    @Override
    public synchronized void connect(final DeviceConnectCallback cb) {
        this.connect(30, cb);
    }

    @Override
    public void connect(final int timeOut, final DeviceConnectCallback cb) {
        if (gatt != null) {
            cb.onError(this, new DeviceCallbackException(Device.ERROR_ALREADY_CONNECTED));
            return;
        }
        isServiceDiscovering = false;
        this.connectCallback = cb;


        gatt = bluetoothDevice.connectGatt(result.getContext(), false, getBluetoothGattCallback());
        ALog.d(TAG, String.format("connect device of [%s - %s]", bluetoothDevice.getAddress(), bluetoothDevice.getName()));
        BLEDevice.this.state = BLE_CONNECT_STATE_CONNECTING;
        fireEvent(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));

        //注册蓝牙状态改变系统广播监听
        broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (BluetoothAdapter.ACTION_STATE_CHANGED.equals(intent.getAction())) {
                    int state = intent.getIntExtra(BluetoothAdapter.EXTRA_STATE, BluetoothAdapter.STATE_OFF);
                    if (state == BluetoothAdapter.STATE_ON) {
                        onBluetoothStateON();
                    } else if (state == BluetoothAdapter.STATE_OFF) {
                        onBluetoothStateOFF();
                    }
                }
            }
        };
        // 注册广播监听
        AdvanproAndroidSDK.getAppContext().registerReceiver(broadcastReceiver, new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED));


        timer = new Timer();
        //连接超时定时器
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (BLEDevice.this.state == BLE_CONNECT_STATE_CONNECTING) {
                    ALog.d(TAG, String.format("[%s] connect device of %s is time out %ds and state=%d", bluetoothDevice.getAddress(), bluetoothDevice.getName(), timeOut, state));
                    disconnect(new DeviceDisconnectCallback() {
                        @Override
                        public void onSucceed(Device device) {
                            cb.onError(device, new DeviceCallbackException(ERROR_IS_TIME_OUT));
                        }

                        @Override
                        public void onError(Device device, DeviceCallbackException error) {
                            cb.onError(device, new DeviceCallbackException(ERROR_IS_TIME_OUT));
                        }
                    });
                }

            }
        }, timeOut * 1000);

    }

    @Override
    public synchronized void disconnect(DeviceDisconnectCallback cb) {
        this.disconnectCallback = cb;

        if (gatt == null) {
            //抛出未打开连接异常
            Log.i(TAG, String.format("The device %s has been disconnected.", bluetoothDevice.getAddress()));
            cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_DISCONNECTED));
            return;
        }

        ALog.d(TAG, String.format("[%s] disconnect state = %d", bluetoothDevice.getAddress(), state));

        // 如果当前状态不是已经连接，直接释放连接对象。
        if (BLEDevice.this.state != BLE_CONNECT_STATE_CONNECTED) {
            gatt.disconnect();
            afterDisconnectedClearCommand();
            destroyConnect();
            BLEDevice.this.state = BLE_CONNECT_STATE_DISCONNECTED;
            disconnectCallback.onSucceed(BLEDevice.this);
        }

        // 如果当前状态已连接,改为正在断开连接状态
        if (BLEDevice.this.state == BLE_CONNECT_STATE_CONNECTED) {
            gatt.disconnect();
            BLEDevice.this.state = BLE_CONNECT_STATE_DISCONNECTING;
        }

        //发送连接状态改变事件
        fireEvent(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));


    }

    /**
     * 系统蓝牙关闭操作
     */
    protected void onBluetoothStateOFF() {
        ALog.d(TAG, "bluetooth off");
        synchronized (BLEDevice.this) {
            //如果当前设备已经断开,不用处理
            if (BLEDevice.this.state == BLE_CONNECT_STATE_DISCONNECTED) {
                return;
            }
            int oldState = BLEDevice.this.state;
            //蓝牙关闭，直接将状态改为已断开连接，并销毁连接
            BLEDevice.this.state = BLE_CONNECT_STATE_DISCONNECTED;
            afterDisconnectedClearCommand();
            destroyConnect();

            //如果之前设备正在连接中，发送连接连接失败消息
            if (oldState == BLE_CONNECT_STATE_CONNECTING) {
                connectCallback.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_DISCONNECTED));
            }

            //如果之前设备正在断开连接,发送已成功断开消息
            if (oldState == BLE_CONNECT_STATE_DISCONNECTING) {
                disconnectCallback.onSucceed(BLEDevice.this);
            }

            //发送连接状态改变事件
            fireEvent(EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));
        }
    }

    /**
     * 系统蓝牙开启
     */
    protected void onBluetoothStateON() {
        ALog.d(TAG, "bluetooth on");
    }

    /**
     * 销毁蓝牙连接、定时器
     */
    private void destroyConnect() {
        ALog.d(TAG, "destroy connect");
        //销毁广播监听
        if (broadcastReceiver != null)
            AdvanproAndroidSDK.getAppContext().unregisterReceiver(broadcastReceiver);
        broadcastReceiver = null;

        //关闭、销毁定时器
        if (timer != null)
            timer.cancel();
        timer = null;

        //关闭、销毁蓝牙gatt
        if (gatt != null) {
            refreshGatt();
            gatt.close();
        }
        gatt = null;

    }

    /**
     * 清除命令请求
     */
    private void afterDisconnectedClearCommand() {
        synchronized (queue) {
            BLEDevice.this.queue.clear();
        }
        synchronized (commandCallbackMap) {
            for (String key : commandCallbackMap.keySet()) {
                if (commandCallbackMap.get(key) != null) {
                    final DeviceCommandCallback cb = commandCallbackMap.get(key);
                    commandCallbackMap.remove(key);
                    onCommandTimeOut(key);
                    commandExecutors.execute(new Runnable() {
                        @Override
                        public void run() {
                            cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_DISCONNECTED));
                        }
                    });
                }
            }
        }
    }

    private void refreshGatt() {
        try {
            Method localMethod = gatt.getClass().getMethod("refresh");
            if (localMethod != null) {
                localMethod.invoke(gatt);
            }
        } catch (Exception localException) {
            ALog.i(TAG, "An exception occurred while refreshing device");
        }
    }

    @Override
    public void on(String key, DeviceEventCallback cb) {
        synchronized (listeners) {
            if (!listeners.containsKey(key)) {
                listeners.put(key, new LinkedHashSet<DeviceEventCallback>());
            }

            listeners.get(key).add(cb);
        }

    }

    @Override
    public void un(String key, DeviceEventCallback cb) {
        synchronized (listeners) {
            if (listeners.containsKey(key)) {
                listeners.get(key).remove(cb);
            }
        }
    }

    @Override
    public void unAllEvent() {
        synchronized (listeners) {
            listeners.clear();
        }
    }

    @Override
    public synchronized void command(String key, DeviceCommandParams params, DeviceCommandCallback cb) {
        this.command(key, params, 30, cb);
    }

    @Override
    public void command(String key, DeviceCommandParams params, int timeOut, DeviceCommandCallback cb) {
        BLEDevice.CommandChannelInfo cmd = getCommandChannelInfo(key);
        if (cmd != null && registerCommand(cmd, timeOut, cb)) {
            write(cmd.getService(), cmd.getCharacteristic(), params.toBytes());
            afterRegisterCommand(cmd.getKey(), cb);
        }
    }

    @Override
    public boolean isConnected() {
        return state == BLE_CONNECT_STATE_CONNECTED;
    }

    @Override
    public boolean isClose() {
        return gatt == null;
    }

    protected BluetoothGattCharacteristic getBluetoothGattCharacteristic(UUID service, UUID characteristic) {
        return gatt.getService(service) != null
                ? gatt.getService(service).getCharacteristic(characteristic) : null;
    }

    protected boolean registerCommand(BLEDevice.CommandChannelInfo commandInfo, final int timeOut, final DeviceCommandCallback cb) {
        final String key = commandInfo.getKey();
        synchronized (commandCallbackMap) {
            if (commandCallbackMap.containsKey(key)) {
                commandExecutors.execute(new Runnable() {
                    @Override
                    public void run() {
                        cb.onError(BLEDevice.this, new DeviceCallbackException(Device.ERROR_LAST_COMMAND_NOT_DONE, "key=" + key));
                    }
                });
                return false;
            }
        }

        if (gatt == null) {
            commandExecutors.execute(new Runnable() {
                @Override
                public void run() {
                    cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_DISCONNECTED, "key=" + key));
                }
            });
            return false;
        }

        if (gatt.getService(commandInfo.getService()) == null) {
            commandExecutors.execute(new Runnable() {
                @Override
                public void run() {
                    cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_SERVICE_NOT_EXISTS, "key=" + key));
                }
            });
            return false;
        }
        if (gatt.getService(commandInfo.getService()).getCharacteristic(commandInfo.getCharacteristic()) == null) {
            commandExecutors.execute(new Runnable() {
                @Override
                public void run() {
                    cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_CHARACTERISTIC_NOT_EXISTS, "key=" + key));
                }
            });
            return false;
        }
        synchronized (commandCallbackMap) {
            commandCallbackMap.put(key, cb);
        }

        if (timeOut > 0) {
            //超时定时任务
            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    synchronized (commandCallbackMap) {
                        if (commandCallbackMap.containsKey(key) && commandCallbackMap.get(key).equals(cb)) {
                            onCommandTimeOut(key);
                            commandCallbackMap.remove(key);
                            cb.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_TIME_OUT, "key=" + key, "timeOut=" + timeOut + "s"));
                        }
                    }
                }
            }, timeOut * 1000);
        }
        return true;
    }

    public boolean isFindService(UUID serviceUUId) {
        if (gatt == null)
            return false;
        return gatt.getService(serviceUUId) != null;
    }

    public boolean isFindCharacteristic(UUID serviceUUId, UUID characteristicUUId) {
        if (gatt == null)
            return false;
        return gatt.getService(serviceUUId) != null && gatt.getService(serviceUUId).getCharacteristic(characteristicUUId) != null;
    }

    protected void fireCommand(String key, final DeviceCommandResult result) {
        synchronized (commandCallbackMap) {
            if (commandCallbackMap.get(key) != null) {
                final DeviceCommandCallback cb = commandCallbackMap.get(key);
                commandCallbackMap.remove(key);
                commandExecutors.execute(new Runnable() {
                    @Override
                    public void run() {
                        cb.onSuccess(BLEDevice.this, result);
                    }
                });
            }
        }

    }

    protected void fireEvent(final String key, final DeviceEventResult result) {
        if (!listeners.containsKey(key)) {
            return;
        }

        eventExecutor.execute(new Runnable() {
            @Override
            public void run() {
                Set<DeviceEventCallback> set = listeners.get(key);
                for (DeviceEventCallback cb : set) {
                    cb.onData(BLEDevice.this, result);
                }

            }
        });
    }

    public void notify(UUID service, UUID characteristic, boolean enable) {
        synchronized (queue) {
            ALog.d(TAG, String.format("[%s] notify %s & %s [%s]", bluetoothDevice.getAddress(), service.toString(), characteristic.toString(), String.valueOf(enable)));
            BLEDevice.SerialQueueItem item = new BLEDevice.SerialQueueItem(service, characteristic, null, enable ? 1 : 2, 0);
            queue.add(item);
            if (queue.size() == 1) {
                notification(item);
            }
        }
    }

    protected void write(UUID service, UUID characteristic, byte[] data) {
        if (gatt == null) {
            return;
        }
        synchronized (queue) {
            List<BLEDevice.SerialQueueItem> items = new ArrayList<>();
            boolean startNow = queue.isEmpty();

            if (data.length > 20) {
                ByteBuffer buffer = ByteBuffer.wrap(data);
                while (buffer.remaining() > 0) {
                    byte[] bytes = new byte[buffer.remaining() >= 20 ? 20 : buffer.remaining()];
                    buffer.get(bytes);

                    queue.add(new BLEDevice.SerialQueueItem(service, characteristic, bytes, 0, queue.size()));
                }
            } else {
                queue.add(new BLEDevice.SerialQueueItem(service, characteristic, data, 0, queue.size()));
            }

            ALog.d(TAG, String.format("[%s] write data to %s & %s [%s]", bluetoothDevice.getAddress(), service.toString(), characteristic.toString(), CommonUtils.bytesToHexString(data)));
            if (startNow) {
                output(queue.peek());
            }
        }
    }


    protected void read(UUID service, UUID characteristic) {
        if (gatt == null) {
            return;
        }
        synchronized (queue) {
            ALog.d(TAG, String.format("[%s] read data to %s & %s", bluetoothDevice.getAddress(), service.toString(), characteristic.toString()));
            BLEDevice.SerialQueueItem item = new BLEDevice.SerialQueueItem(service, characteristic, null, 0, queue.size());
            queue.add(item);
            if (queue.size() == 1) {
                input(item);
            }
        }
    }

    private boolean notification(BLEDevice.SerialQueueItem item) {
        if (gatt == null) {
            return false;
        }

        BluetoothGattCharacteristic bluetoothGattCharacteristic =
                getBluetoothGattCharacteristic(item.getService(), item.getCharacteristic());

        if (bluetoothGattCharacteristic != null && gatt.setCharacteristicNotification(bluetoothGattCharacteristic, item.getNotify() == 1)) {
            BluetoothGattDescriptor descriptor = bluetoothGattCharacteristic.getDescriptor(CLIENT_CHARACTERISTIC_CONFIG);

            if (descriptor == null) {
                return false;
            }

            if (item.getNotify() == 1) {
                descriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
            } else {
                descriptor.setValue(BluetoothGattDescriptor.DISABLE_NOTIFICATION_VALUE);
            }

            boolean effect = gatt.writeDescriptor(descriptor);
            if (effect) {
                ALog.i(TAG, String.format("[%s] %s notification by %s & %s is successfully", bluetoothDevice.getAddress(), (item.getNotify() == 1 ? "enable" : "disable"), item.getService(), item.getCharacteristic()));
            } else {
                ALog.e(TAG, String.format("[%s] %s notification by %s & %s is failed", bluetoothDevice.getAddress(), (item.getNotify() == 1 ? "enable" : "disable"), item.getService(), item.getCharacteristic()));
            }

            return effect;
        }

        return false;
    }

    private boolean output(BLEDevice.SerialQueueItem item) {
        BluetoothGattCharacteristic bluetoothGattCharacteristic =
                getBluetoothGattCharacteristic(item.getService(), item.getCharacteristic());

        ALog.d(TAG, String.format("[%s] send data to %s & %s [%s]", bluetoothDevice.getAddress(), item.getService().toString(), item.getCharacteristic().toString(), CommonUtils.bytesToHexString(item.getData())));

        return gatt != null
                && item.getData() != null
                && bluetoothGattCharacteristic != null
                && bluetoothGattCharacteristic.setValue(item.getData())
                && gatt.writeCharacteristic(bluetoothGattCharacteristic);

    }

    private boolean input(BLEDevice.SerialQueueItem item) {
        BluetoothGattCharacteristic bluetoothGattCharacteristic =
                getBluetoothGattCharacteristic(item.getService(), item.getCharacteristic());
        boolean result = gatt != null
                && bluetoothGattCharacteristic != null
                && gatt.readCharacteristic(bluetoothGattCharacteristic);
        if (result) {
            ALog.i(TAG, String.format("[%s] read by %s & %s is successfully", bluetoothDevice.getAddress(), item.getService(), item.getCharacteristic()));
        } else {
            ALog.e(TAG, String.format("[%s] read by %s & %s is failed", bluetoothDevice.getAddress(), item.getService(), item.getCharacteristic()));
        }
        return result;
    }

    private boolean next() {
        synchronized (queue) {
            queue.poll();
            BLEDevice.SerialQueueItem item = queue.peek();
            if (item != null) {
                ALog.d(TAG, "[" + bluetoothDevice.getAddress() + "] queue current has " + queue.size() + ". it's will next at " + item.getService() + " & " + item.getCharacteristic());
                if (item.getNotify() == 0) {
                    return item.getData() != null ? output(item) : input(item);
                } else {
                    return notification(item);
                }
            } else {
                return false;
            }

        }
    }

    /**
     * 连接成功或失败就绪，发送连接状态改变事件
     *
     * @param error error==null 表示连接成功，否则连接失败
     */
    protected void fireConnected(final DeviceCallbackException error) {
        synchronized (BLEDevice.this) {

            int oldState = BLEDevice.this.state;

            //修改连接状态
            BLEDevice.this.state = error == null ? BLE_CONNECT_STATE_CONNECTED : BLE_CONNECT_STATE_DISCONNECTED;

            //连接成功,如果连接中，回调连接成功方法
            if (error == null && oldState == BLE_CONNECT_STATE_CONNECTING) {
                connectCallback.onSucceed(BLEDevice.this, null);
            }

            //连接失败如果连接中，销毁连接，并回调连接失败回调
            if (error != null && oldState == BLE_CONNECT_STATE_CONNECTING) {
                gatt.disconnect();
                afterDisconnectedClearCommand();
                destroyConnect();
                connectCallback.onError(BLEDevice.this, error);
            }

            //发送连接状态改变事件
            fireEvent(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));
            ALog.d(TAG, String.format("[%s] fire connect oldState = %d ,newState = %d", bluetoothDevice.getAddress(), oldState, BLEDevice.this.state));

        }

    }

    @Override
    public String getInfo(String key) {
        return info.get(key);
    }

    public void setInfo(String key, String value) {
        info.put(key, value);
    }

    abstract protected BluetoothGattCallback getBluetoothGattCallback();

    abstract protected CommandChannelInfo getCommandChannelInfo(String key);

    abstract protected void afterRegisterCommand(String key, final DeviceCommandCallback cb);

    abstract protected void onCommandTimeOut(String key);

    public BLEDeviceScanResult getScanResult() {
        return result;
    }

    @Override
    public void setEnableReConnect(boolean enable) {
        this.enableReconnect = enable;
    }

    private static class SerialQueueItem implements Comparable<SerialQueueItem> {
        private UUID service;

        private UUID characteristic;

        private byte[] data;

        private int notify;

        private int priority;


        public SerialQueueItem(UUID service, UUID characteristic, byte[] data, int notify, int priority) {
            this.service = service;
            this.characteristic = characteristic;
            this.data = data;
            this.notify = notify;
            this.priority = priority;
        }

        public UUID getService() {
            return service;
        }

        public void setService(UUID service) {
            this.service = service;
        }

        public UUID getCharacteristic() {
            return characteristic;
        }

        public void setCharacteristic(UUID characteristic) {
            this.characteristic = characteristic;
        }

        public byte[] getData() {
            return data;
        }

        public void setData(byte[] data) {
            this.data = data;
        }

        public int getNotify() {
            return notify;
        }

        public void setNotify(int notify) {
            this.notify = notify;
        }

        public int getPriority() {
            return priority;
        }

        public void setPriority(int priority) {
            this.priority = priority;
        }

        @Override
        public int compareTo(@NonNull SerialQueueItem o) {
            return getPriority() - o.getPriority();
        }
    }

    protected static class CommandChannelInfo {

        private String key;

        private UUID service;

        private UUID characteristic;

        public CommandChannelInfo(String key, UUID service, UUID characteristic) {

            this.key = key;
            this.service = service;
            this.characteristic = characteristic;
        }

        public String getKey() {
            return key;
        }

        public UUID getService() {
            return service;
        }

        public UUID getCharacteristic() {
            return characteristic;
        }
    }

    public class BLEDeviceBluetoothGattCallback extends BluetoothGattCallback {
        @Override
        public void onConnectionStateChange(final BluetoothGatt gatt, int status, int newState) {
            ALog.d(TAG, String.format("[%s] connect status = %s and state = %s", bluetoothDevice.getAddress(), status, newState));
            synchronized (BLEDevice.this) {
                //记录旧的连接状态
                int oldState = BLEDevice.this.state;

                // 设备连接成功，进行发现服务等操作。
                if (newState == BluetoothProfile.STATE_CONNECTED && (oldState == BLE_CONNECT_STATE_CONNECTING || BLEDevice.this.gatt != null)) {
                    BLEDevice.this.gatt = gatt;

                    //启动发现服务监听进程，进行发现服务重试操作。
                    isServiceDiscovering = true;
                    timer.schedule(new TimerTask() {
                        private int retry = 0;

                        @Override
                        public void run() {
                            while (retry < DISCOVER_SERVICE_RETRY_TIME && BLEDevice.this.isServiceDiscovering) {
                                try {
                                    retry++;
                                    boolean result = gatt.discoverServices();
                                    ALog.d(TAG, String.format("[%s] retry request service discovery result is %s ", bluetoothDevice.getAddress(), result));

                                    Thread.sleep(result ? 5000 : 500);
                                } catch (InterruptedException e) {
                                    ALog.e(TAG, e);
                                }
                            }
                            synchronized (BLEDevice.this) {

                                // 重试#DISCOVER_SERVICE_RETRY_TIME#次后，仍然未发现服务成功，激活发现服务操时的错误事件。
                                if (isServiceDiscovering) {
                                    gatt.disconnect();
                                    afterDisconnectedClearCommand();

                                    //断开连接，需要重连接设备，尝试重新连接
                                    if (enableReconnect && BLEDevice.this.state == BLE_CONNECT_STATE_DISCONNECTED && !isUpgrading()) {
                                        ALog.d(TAG, String.format("try to reconnect %s", bluetoothDevice.getAddress()));
                                        gatt.connect();
                                    }

                                    //如果第一次连接中发现服务失败，销毁连接并回调连接失败函数
                                    if (BLEDevice.this.state == BLE_CONNECT_STATE_CONNECTING) {
                                        destroyConnect();
                                        BLEDevice.this.state = BLE_CONNECT_STATE_DISCONNECTED;
                                        connectCallback.onError(BLEDevice.this, new DeviceCallbackException(ERROR_SERVICE_DISCOVERY_TIMEOUT));
                                        fireEvent(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));
                                    }
                                }
                            }

                        }
                    }, 500);

                }

                // 设备断开连接
                if (newState == BluetoothProfile.STATE_DISCONNECTED && BLEDevice.this.gatt != null && BLEDevice.this.gatt == gatt) {
                    // 将连接状态改为断开连接
                    BLEDevice.this.state = BLE_CONNECT_STATE_DISCONNECTED;

                    afterDisconnectedClearCommand();
                    // 自动断开同时未开启重连、手动断开,连接中断开，自动释放当前连接
                    if (!enableReconnect || oldState == BLE_CONNECT_STATE_DISCONNECTING || oldState == BLE_CONNECT_STATE_CONNECTING) {
                        destroyConnect();
                    }

                    // 如果手动断开，发送已成功断开消息
                    if (oldState == BLE_CONNECT_STATE_DISCONNECTING) {
                        disconnectCallback.onSucceed(BLEDevice.this);
                    }

                    //如果连接中断开，发送连接失败消息
                    if (oldState == BLE_CONNECT_STATE_CONNECTING) {
                        connectCallback.onError(BLEDevice.this, new DeviceCallbackException(ERROR_IS_DISCONNECTED));
                    }

                    // 处理连接状态改变事件通知
                    fireEvent(BLEDevice.EVENT_CONNECT_STATE_CHANGE, new BLEConnectStateChangeResult(BLEDevice.this, BLEDevice.this.state));

                    // 如果开启了重连机制并且不是手动断开,或不是连接中，同时不是升级状态，需要进行重连处理
                    if (enableReconnect && oldState != BLE_CONNECT_STATE_DISCONNECTING && oldState != BLE_CONNECT_STATE_CONNECTING && !isUpgrading()) {
                        ALog.d(TAG, String.format("try to reconnect %s", bluetoothDevice.getAddress()));
                        gatt.connect();
                    }


                }


            }
        }


        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            synchronized (BLEDevice.this) {
                isServiceDiscovering = false;
            }
            ALog.d(TAG, String.format("[%s] on services discovery status is %s ", bluetoothDevice.getAddress(), status));
        }

        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            ALog.d(TAG, String.format("[%s] on characteristic read by %s, it is data [%s] and status is %s", bluetoothDevice.getAddress(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(characteristic.getValue()), status));
            super.onCharacteristicRead(gatt, characteristic, status);
            next();
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            ALog.d(TAG, String.format("[%s] on characteristic write by %s, it is data [%s] and status is %s", bluetoothDevice.getAddress(), characteristic.getUuid().toString(), CommonUtils.bytesToHexString(characteristic.getValue()), status));
            super.onCharacteristicWrite(gatt, characteristic, status);
            next();
        }

        @Override
        public void onDescriptorRead(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorRead(gatt, descriptor, status);
            next();
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            super.onDescriptorWrite(gatt, descriptor, status);
            next();
        }

        @Override
        public void onReadRemoteRssi(BluetoothGatt gatt, int rssi, int status) {
            super.onReadRemoteRssi(gatt, rssi, status);
            Log.d(TAG, String.format("[%s] on rssi change", rssi));
        }
    }
}
